@@ -1,4 +1,4 @@
-# Copyright 2015-2017 Google Inc. All Rights Reserved.
+# Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -49,10 +49,13 @@ class Subtype(object):
   DICTIONARY_KEY_PART = 12
   DICTIONARY_VALUE = 13
   DICT_SET_GENERATOR = 14
+  COMP_EXPR = 21
   COMP_FOR = 15
   COMP_IF = 16
   FUNC_DEF = 17
   DECORATOR = 18
+  TYPED_NAME = 19
+  TYPED_NAME_ARG_LIST = 20
 
 
 class FormatToken(object):
@@ -119,10 +122,11 @@ class FormatToken(object):
       spaces: (int) The number of spaces to place before the token.
       indent_level: (int) The indentation level.
     """
-    indent_char = '\t' if style.Get('USE_TABS') else ' '
-    token_indent_char = indent_char if newlines_before > 0 else ' '
-    indent_before = (indent_char * indent_level * style.Get('INDENT_WIDTH') +
-                     token_indent_char * spaces)
+    if style.Get('USE_TABS'):
+      indent_before = '\t' * indent_level + ' ' * spaces
+    else:
+      indent_before = (
+          ' ' * indent_level * style.Get('INDENT_WIDTH') + ' ' * spaces)
 
     if self.is_comment:
       comment_lines = [s.lstrip() for s in self.value.splitlines()]
@@ -145,9 +149,14 @@ class FormatToken(object):
   def RetainHorizontalSpacing(self, first_column, depth):
     """Retains a token's horizontal spacing."""
     previous = self.previous_token
-    if previous is None:
+    if not previous:
       return
 
+    if previous.is_pseudo_paren:
+      previous = previous.previous_token
+      if not previous:
+        return
+
     cur_lineno = self.lineno
     prev_lineno = previous.lineno
     if previous.is_multiline_string:
@@ -172,6 +181,7 @@ class FormatToken(object):
       prev_len = len(previous.value.split('\n')[-1])
       if '\n' in previous.value:
         prev_column = 0  # Last line starts in column 0.
+
     self.spaces_required_before = cur_column - (prev_column + prev_len)
 
   def OpensScope(self):
@@ -263,9 +273,17 @@ class FormatToken(object):
   @property
   @py3compat.lru_cache()
   def is_multiline_string(self):
+    if py3compat.PY3:
+      prefix = "("
+      prefix += "r|u|R|U|f|F|fr|Fr|fR|FR|rf|rF|Rf|RF"  # strings
+      prefix += "|b|B|br|Br|bR|BR|rb|rB|Rb|RB"  # bytes
+      prefix += ")?"
+    else:
+      prefix = "[uUbB]?[rR]?"
+
+    regex = r'^{prefix}(?P<delim>"""|\'\'\').*(?P=delim)$'.format(prefix=prefix)
     return (self.is_string and
-            re.match(r'^[uUbB]?[rR]?(?P<delim>"""|\'\'\').*(?P=delim)$',
-                     self.value, re.DOTALL) is not None)
+            re.match(regex, self.value, re.DOTALL) is not None)
 
   @property
   @py3compat.lru_cache()
