@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2017 Google Inc. All Rights Reserved.
+# Copyright 2015 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -278,6 +278,43 @@ class FormatFileTest(unittest.TestCase):
       formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
       self.assertCodeEqual(code, formatted_code)
 
+  def testSplittingSemicolonStatements(self):
+    unformatted_code = textwrap.dedent(u"""\
+        def f():
+          x = y + 42 ; z = n * 42
+          if True: a += 1 ; b += 1; c += 1
+        """)
+    expected_formatted_code = textwrap.dedent(u"""\
+        def f():
+            x = y + 42
+            z = n * 42
+            if True:
+                a += 1
+                b += 1
+                c += 1
+        """)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(expected_formatted_code, formatted_code)
+
+  def testSemicolonStatementsDisabled(self):
+    unformatted_code = textwrap.dedent(u"""\
+        def f():
+          x = y + 42 ; z = n * 42  # yapf: disable
+          if True: a += 1 ; b += 1; c += 1
+        """)
+    expected_formatted_code = textwrap.dedent(u"""\
+        def f():
+            x = y + 42 ; z = n * 42  # yapf: disable
+            if True:
+                a += 1
+                b += 1
+                c += 1
+        """)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(expected_formatted_code, formatted_code)
+
   def testDisabledSemiColonSeparatedStatements(self):
     code = textwrap.dedent(u"""\
         # yapf: disable
@@ -346,7 +383,11 @@ class CommandLineTest(unittest.TestCase):
   def tearDownClass(cls):
     shutil.rmtree(cls.test_tmpdir)
 
-  def assertYapfReformats(self, unformatted, expected, extra_options=None):
+  def assertYapfReformats(self,
+                          unformatted,
+                          expected,
+                          extra_options=None,
+                          env=None):
     """Check that yapf reformats the given code as expected.
 
     Invokes yapf in a subprocess, piping the unformatted code into its stdin.
@@ -356,16 +397,18 @@ class CommandLineTest(unittest.TestCase):
       unformatted: unformatted code - input to yapf
       expected: expected formatted code at the output of yapf
       extra_options: iterable of extra command-line options to pass to yapf
+      env: dict of environment variables.
     """
     cmdline = YAPF_BINARY + (extra_options or [])
     p = subprocess.Popen(
         cmdline,
         stdout=subprocess.PIPE,
         stdin=subprocess.PIPE,
-        stderr=subprocess.PIPE)
+        stderr=subprocess.PIPE,
+        env=env)
     reformatted_code, stderrdata = p.communicate(unformatted.encode('utf-8'))
     self.assertEqual(stderrdata, b'')
-    self.assertEqual(reformatted_code.decode('utf-8'), expected)
+    self.assertMultiLineEqual(reformatted_code.decode('utf-8'), expected)
 
   def testUnicodeEncodingPipedToFile(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -492,7 +535,10 @@ class CommandLineTest(unittest.TestCase):
         suffix='.py', dirname=self.test_tmpdir) as (out, _):
       with utils.TempFileContents(
           self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
-        subprocess.check_call(YAPF_BINARY + ['--diff', filepath], stdout=out)
+        try:
+          subprocess.check_call(YAPF_BINARY + ['--diff', filepath], stdout=out)
+        except subprocess.CalledProcessError as e:
+          self.assertEqual(e.returncode, 1)  # Indicates the text changed.
 
   def testReformattingSpecificLines(self):
     unformatted_code = textwrap.dedent("""\
@@ -775,6 +821,8 @@ class CommandLineTest(unittest.TestCase):
         """)
     expected_formatted_code = textwrap.dedent("""\
         a = line_to_format
+
+
         def f():
             x = y + 42; z = n * 42
             if True: a += 1 ; b += 1 ; c += 1
@@ -797,6 +845,8 @@ class CommandLineTest(unittest.TestCase):
         ''')
     expected_formatted_code = textwrap.dedent('''\
         foo = 42
+
+
         def f():
             email_text += """<html>This is a really long docstring that goes over the column limit and is multi-line.<br><br>
         <b>Czar: </b>"""+despot["Nicholas"]+"""<br>
@@ -1010,10 +1060,12 @@ class CommandLineTest(unittest.TestCase):
 
   def testCoalesceBrackets(self):
     unformatted_code = textwrap.dedent("""\
-       some_long_function_name_foo({
-           'first_argument_of_the_thing': id,
-           'second_argument_of_the_thing': "some thing"}
-           )""")
+       some_long_function_name_foo(
+           {
+               'first_argument_of_the_thing': id,
+               'second_argument_of_the_thing': "some thing"
+           }
+       )""")
     expected_formatted_code = textwrap.dedent("""\
        some_long_function_name_foo({
            'first_argument_of_the_thing': id,
@@ -1024,7 +1076,6 @@ class CommandLineTest(unittest.TestCase):
       f.write(
           textwrap.dedent(u'''\
           [style]
-          based_on_style = facebook
           column_limit=82
           coalesce_brackets = True
           '''))
@@ -1042,6 +1093,7 @@ class CommandLineTest(unittest.TestCase):
         """)
     expected_formatted_code = textwrap.dedent("""\
         def foo():
+
           def bar():
             return {msg_id: author for author, msg_id in reader}
         """)
@@ -1105,22 +1157,46 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--lines', '1-1', '--style', 'chromium'])
 
   def testUseTabs(self):
-    unformatted_code = textwrap.dedent("""\
-        def foo_function():
-         if True:
-          pass
-        """)
-    expected_formatted_code = textwrap.dedent("""\
-        def foo_function():
-        	if True:
-        		pass
-        """)
-    style_contents = textwrap.dedent(u'''\
-        [style]
-        based_on_style = chromium
-        USE_TABS = true
-        INDENT_WIDTH=1
-        ''')
+    unformatted_code = """\
+def foo_function():
+ if True:
+  pass
+"""
+    expected_formatted_code = """\
+def foo_function():
+	if True:
+		pass
+"""
+    style_contents = u"""\
+[style]
+based_on_style = chromium
+USE_TABS = true
+INDENT_WIDTH=1
+"""
+    with utils.TempFileContents(self.test_tmpdir, style_contents) as stylepath:
+      self.assertYapfReformats(
+          unformatted_code,
+          expected_formatted_code,
+          extra_options=['--style={0}'.format(stylepath)])
+
+  def testUseTabsWith(self):
+    unformatted_code = """\
+def f():
+  return ['hello', 'world',]
+"""
+    expected_formatted_code = """\
+def f():
+	return [
+	    'hello',
+	    'world',
+	]
+"""
+    style_contents = u"""\
+[style]
+based_on_style = chromium
+USE_TABS = true
+INDENT_WIDTH=1
+"""
     with utils.TempFileContents(self.test_tmpdir, style_contents) as stylepath:
       self.assertYapfReformats(
           unformatted_code,
@@ -1151,6 +1227,77 @@ class CommandLineTest(unittest.TestCase):
           expected_formatted_code,
           extra_options=['--style={0}'.format(stylepath)])
 
+  def testSpacingBeforeComments(self):
+    unformatted_code = textwrap.dedent("""\
+        A = 42
+
+
+        # A comment
+        def x():
+            pass
+        def _():
+            pass
+        """)
+    expected_formatted_code = textwrap.dedent("""\
+        A = 42
+
+
+        # A comment
+        def x():
+            pass
+        def _():
+            pass
+        """)
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-2'])
+
+  def testSpacingBeforeCommentsInDicts(self):
+    unformatted_code = textwrap.dedent("""\
+        A=42
+
+        X = {
+            # 'Valid' statuses.
+            PASSED:  # Passed
+                'PASSED',
+            FAILED:  # Failed
+                'FAILED',
+            TIMED_OUT:  # Timed out.
+                'FAILED',
+            BORKED:  # Broken.
+                'BROKEN'
+        }
+        """)
+    expected_formatted_code = textwrap.dedent("""\
+        A = 42
+
+        X = {
+            # 'Valid' statuses.
+            PASSED:  # Passed
+                'PASSED',
+            FAILED:  # Failed
+                'FAILED',
+            TIMED_OUT:  # Timed out.
+                'FAILED',
+            BORKED:  # Broken.
+                'BROKEN'
+        }
+        """)
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--style', 'chromium', '--lines', '1-1'])
+
+  @unittest.skipUnless(py3compat.PY36, 'Requires Python 3.6')
+  def testCP936Encoding(self):
+    unformatted_code = 'print("中文")\n'
+    expected_formatted_code = 'print("中文")\n'
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        env={'PYTHONIOENCODING': 'cp936'})
+
 
 class BadInputTest(unittest.TestCase):
   """Test yapf's behaviour when passed bad input."""
